------------------------------------------------------------------------------
--  Thin Ada95 binding to OCI (Oracle Call Interface)                    --
--  Copyright (C) 2000-2002 Dmitriy Anisimkov.                              --
--  License agreement and authors contact information are in file oci.ads   --
------------------------------------------------------------------------------

--  $Id$

with
--   System,
   --   Ada.Task_Identification,
   --   Ada.Strings,
   OCI.Thread,
   Ada.Strings.Fixed;

with OCI.Environments;

--with Ada.Text_IO, System.Address_Image;

package body OCI.Thick.Connections is
   use Lib;
   use type Sword;
   use type Ub4;
   use type OCIHandle;

   procedure Destroy (Object : in out Connection) is
      Rc : Sword;
      H : OCIHandle := Object.Handle;
   begin
      if H /= Empty_Handle then
         if Object.Sessn /= OCISession (Empty_Handle) then
            Rc := OCISessionEnd (Svchp => OCISvcCtx (H),
                                errhp => Thread.Error,
                                usrhp => Object.Sessn);
            Free (OCIHandle(Object.Sessn), OCI_HTYPE_SESSION);
            Free (H, OCI_HTYPE_SVCCTX);
         else
--            Ada.Text_IO.Put_Line ("Logoff " & System.Address_Image (H));
            Rc := OCILogoff (Svchp => OCISvcCtx (H),
               Errhp => Thread.Error);
--            Ada.Text_IO.Put_Line ("Logoff done " & SWord'Image(Rc));
           --  Check_Error(Rc); -- 8i bug
         end if;
      end if;
   end;

   -----------
   -- Break --
   -----------

   procedure Break (It : Connection)
   is
      Rc : SWord := OCIBreak (Handle (It), Thread.Error);
   begin
      Check_Error (RC);
   end Break;
   
   -----------
   -- Reset --
   -----------

   procedure Reset (It : Connection)
   is
      Rc : SWord := OCIReset (Handle (It), Thread.Error);
   begin
      Check_Error (RC);
   end Reset;

   -----------
   -- Logon --
   -----------

   function Logon(Server_Name : String; User : String; Password : String) 
      return Connection is
      Rc : Sword;
      H : aliased OCISvcCtx;
      Result : Connection;
   begin
      Rc := OCILogon(Envhp => Thread.Environment,
         Errhp => Thread.Error,
         Svchp => H'access,
         Username => C.To_C (User),
         Uname_Len => User'Length,
         Password => C.To_C (Password),
         Passwd_Len => Password'Length,
         Dbname => C.To_C (Server_Name),
         Dbname_Len => Server_Name'Length);
      Check_Error(Rc);
      Result.Handle := OCIHandle(H);
      Result.Environment := Thread.Synch.Environment;
      return Result;
   end Logon;

   function Logon (DB : Server;
      User : String; Password : String) return Connection
   is
      Result : Connection;
   begin
      Result.Environment := Thread.Synch.Environment;
      Result.DB := DB;
      Result.Handle := Alloc_Handle (Thread.Environment, OCI_HTYPE_SVCCTX);
      Set_Attr (Result.Handle, OCI_HTYPE_SVCCTX, Handle (DB), OCI_ATTR_SERVER);
      Result.Sessn := OCISession (Alloc_Handle (Thread.Environment, OCI_HTYPE_SESSION));
      Set_Attr (OCIHandle (Result.Sessn), OCI_HTYPE_SESSION, User, OCI_ATTR_USERNAME);
      Set_Attr (OCIHandle (Result.Sessn), OCI_HTYPE_SESSION, Password, OCI_ATTR_PASSWORD);
      Set_Attr (Result.Handle, OCI_HTYPE_SVCCTX, OCIHandle (Result.Sessn), OCI_ATTR_SESSION);
      Check_Error (OCISessionBegin (OCISvcCtx (Result.Handle), Thread.Error, Result.Sessn));
      return Result;
   end Logon;
   

   function Logon (Connect : String) return Connection
         is
      D1,D2 : Natural;
      use Ada.Strings.Fixed,Ada.Strings;
   begin
      D1 := Index(Connect, "/");
      D2 := Index(Connect, "@", Backward);
      if D2=0 then
         return Logon (Server_Name => "",
            User => Connect(Connect'First..D1-1),
            Password => Connect(D1+1..Connect'Last));
      else
         return Logon (
            Server_Name => Connect(D2+1..Connect'Last),
            User => Connect(Connect'First..D1-1),
            Password => Connect(D1+1..D2-1));
      end if;
   end Logon;

   procedure Logoff (Connect : in out Connection) is
      Result : Connection;
   begin
      Connect := Result;
   end Logoff;

   function Server_Version(Connect : in Connection) 
      return String is
      Buff : aliased Text := (0..511 => c.nul);
      rc : sword := OCIServerVersion (
               hndlp => Connect.Handle, 
               errhp => Thread.Error,
               bufp  => CStr.To_Chars_Ptr(Buff'Unchecked_Access),
               bufsz  => buff'Length-1,
               hndltype => OCI_HTYPE_SVCCTX);
   begin
     Check_Error (rc);
     return C.To_Ada(Buff);
   end Server_Version;

   procedure Commit(Connect : in Connection) is
     rc : sword := OCITransCommit(
          OCISvcCtx(Connect.Handle),Thread.Error);
   begin
     Check_Error(rc);
   end;
   
   procedure Rollback(Connect : in Connection) is
     rc : sword := OCITransRollback(
        OCISvcCtx(Connect.Handle),Thread.Error);
   begin
     Check_Error(rc);
   end;


   -- get Connection for extproc callbacks
   function Get_Connection (Context: OCIExtProcContext) 
      return Connection is
      Rc : Sword;
      H : aliased OCISvcCtx;
      E : aliased OCIEnv;
      Er: aliased OCIError;
      tr: OCI.Environments.Thread_Environment;
      Result : Connection;
   begin
   
      Rc := OCIExtProcGetEnv (
         octxp => Context,
         Envhp => E'access,
         Errhp => Er'access,
         Svchp => H'access);
         
      tr := (RF.Controlled_Reference with Handle => E);
      -- Thread.Set_Environment(tr);
      
      Thread.Set_Error (E, Er);

      Check_Error (Rc);
      
      Result.Handle := OCIHandle(H);
      Result.Environment := Thread.Synch.Environment;

      return Result;
      
   end Get_Connection;

end OCI.Thick.Connections;

