------------------------------------------------------------------------------
--  Ada95 Interface to Oracle RDBMS                                         --
--  Copyright (C) 2000-2001 Dmitriy Anisimkov.                              --
--  License text and authors contact information could be found in oci.ads  --
------------------------------------------------------------------------------
with
   Ada.Unchecked_Conversion,
   OCI.Thread;

package body OCI.Thick.Parameters is
   use Lib;
   use type Sword;
   use type Ub4;
   use type OCIHandle;

   function to_typecode is new Ada.Unchecked_Conversion(Integer, Typecode);
   
   procedure Free(Param : OCIParam) is
     rc : sword;
   begin
     if Param=OCIParam(Empty_Handle) then
       return;
     end if;
--     Put("PARAM ");
     rc := OCIDescriptorFree (
         descp => OCIHandle(Param),
         dtype => OCI_DTYPE_PARAM);
--     Put_Line("freed "&SWord'Image(rc));
     Check_Error(rc);
   end Free;

   procedure Destroy  (Object : in out Parameter) is
     H : OCIHandle := Object.Handle;
   begin
     Free(OCIParam(H));
   end;    

   function Type_Name(Param : Parameter) return String is
      H : OCIHandle := Param.Handle;
   begin
      if H=Empty_Handle then
         return "";
      else
         return Get_Attr(H,OCI_DTYPE_PARAM,OCI_ATTR_TYPE_NAME);
      end if;
   end Type_Name;

   function Name(Param : Parameter) return String is
      H : OCIHandle := Param.Handle;
   begin
      if H=Empty_Handle then
         return "";
      else
         return Get_Attr (H,OCI_DTYPE_PARAM,OCI_ATTR_NAME);
      end if;
   end Name;
 
   function Type_Code(Param : Parameter) return Typecode is
   begin
      return to_Typecode(Get_Attr(
         Param.Handle,OCI_DTYPE_PARAM,OCI_ATTR_DATA_TYPE));
   end Type_Code;

   function Data_Size(Param : Parameter) return Natural is
   begin
      return Get_Attr (Param.Handle,OCI_DTYPE_PARAM,OCI_ATTR_DATA_SIZE);
   end;
   
   function PRECISION(Param : Parameter) return Integer is
   begin
     return Get_Attr (Param.Handle,OCI_DTYPE_PARAM,OCI_ATTR_PRECISION);
   end;

   function SCALE(Param : Parameter) return Integer is
   begin
     return Get_Attr(Param.Handle,OCI_DTYPE_PARAM,OCI_ATTR_SCALE);
   end;
   
   function IS_NULL(Param : Parameter) return Boolean is
   begin
     return Get_Attr(Param.Handle,OCI_DTYPE_PARAM,OCI_ATTR_IS_NULL)/=0;
   end;

   function SCHEMA_NAME(Param : Parameter) return String is
   begin
     return Get_Attr(Param.Handle,OCI_DTYPE_PARAM,OCI_ATTR_SCHEMA_NAME);
   end;

   function Get_Parameter(Stmt : Statement; Index : Positive) return
         OCIParam is
      Result : aliased OCIParam;
      Rc : Sword := OCIParamGet (
         Hndlp => Handle(Stmt),
         Htype => OCI_HTYPE_STMT,
         Errhp => Thread.Error,
         Parmdpp => Result'access,
         Pos => Ub4(Index));
   begin
      if Rc=OCI_NO_DATA then
         return OCIParam(Empty_Handle);
      end if;
      Check_Error(Rc);
      return Result;
   end;

   function Column (Stmt : Statement; Position : Positive) return Parameter
   is
   begin
     return (RF.Controlled_Reference with
       Handle => OCIHandle (Get_Parameter (Stmt, Position)),
       Last_Stmt => Stmt);
   end;

   function Get_Parameters (Stmt : Statement) return Parameter_Array is
     Result : Parameter_Array (1 .. Number_Of_Columns (Stmt));
   begin
     for i in Result'Range loop
        Result(i) := Column(Stmt,i);
     end loop;
     return Result;
   end Get_Parameters;

end OCI.Thick.Parameters;

