------------------------------------------------------------------------------
--  Ada95 Interface to Oracle RDBMS                                         --
--  Copyright (C) 2000-2001 Dmitriy Anisimkov.                              --
--  License text and authors contact information could be found in oci.ads  --
------------------------------------------------------------------------------
with OCI.Thick.Connections;
with Ada.Strings.Unbounded;
package OCI.Thick.Statements is

   use Connections;
   use Ada.Strings.Unbounded;

   type Statement is new Handle_Reference with private;
   
   type VStrings is array (Positive range <>) of Unbounded_String;
   
   type Statement_Type is (
      STMT_SELECT,
      STMT_UPDATE,
      STMT_DELETE,
      STMT_INSERT,
      STMT_CREATE,
      STMT_DROP,
      STMT_ALTER,
      STMT_BEGIN,
      STMT_DECLARE);

   function Prepare (Connect : Connection; Code : String)
      return Statement;
      
   -- Undepended from Connection statement stile.
   function Prepare (Code : String) return Statement;

   procedure Execute (Connect : Connection; Stmt : in out Statement);
   procedure Describe (Connect : Connection; Stmt : in out Statement);


   -- Depended from Connection statement stile.
   -- Like ODBC
   procedure Execute (Stmt : in out Statement);
   procedure Execute (Stmt : in out Statement; Success : out Boolean);
   
   procedure Execute_And_Commit (Stmt : in out Statement);
   procedure Execute_And_Commit (Stmt : in out Statement; Success : out Boolean);

   procedure Describe (Stmt : in out Statement);

   procedure Describe (Stmt : in out Statement; Success : out Boolean);
   
   function Execute (Connect : Connection; Code : String) 
       return Statement;
   procedure Execute (Connect : Connection; Code : String);

   function Type_Of_Statement (Stmt : Statement) return Statement_Type;
   function Described (Stmt : Statement) return Boolean;
   function Executed (Stmt : Statement) return Boolean;

   procedure Set_Connection (Stmt : in out Statement; Connect : Connection);
   function Get_Connection (Stmt : Statement) return Connection;
   function Fetch (Stmt : Statement) return Boolean;
   procedure Cancel (Stmt : Statement);

   function Parse_Error_Offset (Stmt : in Statement) return Natural;

   function Number_Of_Columns (Stmt : Statement) return Natural;
   function Bind_Names (Stmt : Statement) return VStrings;

   pragma Inline (Executed, Get_Connection);

private

   use Lib;

   for Statement_Type use (
      Stmt_Select  => OCI_STMT_SELECT,
      STMT_UPDATE  => OCI_STMT_UPDATE ,
      STMT_Delete  => OCI_STMT_DELETE,
      STMT_Insert  => OCI_STMT_INSERT ,
      STMT_Create  => OCI_STMT_CREATE,
      STMT_DROP    => OCI_STMT_DROP  ,
      STMT_ALTER   => OCI_STMT_ALTER ,
      STMT_BEGIN   => OCI_STMT_BEGIN ,
      STMT_DECLARE => OCI_STMT_DECLARE);
    
   for Statement_Type'Size use Integer'Size;

   type Statement is new Handle_Reference with record
      Connect : Connection;
      Executed, Described : Boolean := False;
   end record;
   
   procedure Destroy (Object : in out Statement);

end OCI.Thick.Statements;