------------------------------------------------------------------------------
--  Ada95 Interface to Oracle RDBMS                                         --
--  Copyright (C) 2000-2001 Dmitriy Anisimkov.                              --
--  License text and authors contact information could be found in oci.ads  --
------------------------------------------------------------------------------
with OCI.Thick.Statements;
with Ada.Strings.Unbounded;
with Interfaces.C;
package OCI.Thick.Strings is

   use OCI.Thick.Statements;
   use Ada.Strings.Unbounded;

   type Var_Type is new Limited_Variable with private;

   subtype Variable is Var_Type;

   procedure Bind (Stmt : Statement'Class;
      Value : in out Var_Type; Name : String);

   procedure Bind
     (Stmt     : in     Statement'Class;
      Value    : in out Var_Type;
      Position : in     Positive);

   procedure Define
     (Stmt     : in     Statement'Class;
      Value    : in out Var_Type;
      Position : in     Positive);

   procedure Set_Value (Var : in out Var_Type; Value : String);

   procedure Set_Value (Var : in out Var_Type; Value : Unbounded_String);

   function Get_Value (Var : Var_Type)
      return Unbounded_String;

   function Get_Value (Var : Var_Type) return String;

   function Value (Var : Var_Type) return String
      renames Get_Value;

   function Value (Var : Var_Type) return Unbounded_String
      renames Get_Value;

   function String_Value (Var : Var_Type) return String
      renames Get_Value;

   function Unbounded_String_Value (Var : Var_Type)
      return Unbounded_String renames Get_Value;

   procedure Complete_Last_Value;

private

      use Interfaces.C;

      type Var_Type is new Limited_Variable with record
         Value : Unbounded_String;
         Complete : Boolean := True;
         Position : Positive := 1;
         First_Out_Piece : Boolean := False;
         rcode    : aliased Ub2 := 0;
         alen     : aliased size_t := 0;
         prev_indicator : Sb2;
      end record;

      procedure Finalize  (Object : in out Var_Type);

   
end OCI.Thick.Strings;