------------------------------------------------------------------------------
--  Ada95 Interface to Oracle RDBMS                                         --
--  Copyright (C) 2000-2002 Dmitriy Anisimkov.                              --
--  License text and authors contact information could be found in oci.ads  --
------------------------------------------------------------------------------

-- $Id$

with 

   Interfaces.C,
   
   Ada.Finalization,
   Ada.Task_Attributes,
   
   OCI.Thick.Strings,
   
   OCI.Lib,
   OCI.Environments;
use 
   OCI.Environments,
   OCI.Lib;
   
private package OCI.Thread is 

   function Error return OCIError;
  
   function Environment return OCIEnv;
  
   protected Synch is
   
      function Environment return Thread_Environment;
      --  Returun Environment of current task
      --  every task has the same environment by default.
      
      function Task_Dedicated_Environment return Boolean;
      --  Creates environment for current task only
      --  Should be called before first call to the library
      --  in this thread or never.
      
   end Synch;

  -- initalize from Extproc Context
   procedure Set_Error (Env : OCIEnv; Err : OCIError);
   
   subtype Char_Buffer is Interfaces.C.char_array (0 .. 1023);

   type Value_Ptr is access all OCI.Thick.Strings.Var_Type;

   type Error_Type is new Ada.Finalization.Controlled with record
      Handle       : OCIError := OCIError(Empty_Handle);
      Environment  : Thread_Environment;
      Last_Value   : Value_Ptr;
      Buffer       : aliased Char_Buffer;
   end record;

--   procedure Initialize (Object : in out Error_Type);
--   
--   procedure Adjust (Object : in out Error_Type);
   
   procedure Finalize (Object : in out Error_Type);

   Init_Error : constant Error_Type := 
       (Ada.Finalization.Controlled with 
            Handle       => OCIError (Empty_Handle),
            Environment => Init_Environment,
            Last_Value  => null,
            Buffer      => (others => Interfaces.C.nul));

  package Attributes is new
      Ada.Task_Attributes (Error_Type, Init_Error);
      
  end OCI.Thread;