with Ada.Characters.Handling;
with Ada.Text_IO;
with Ada.Command_Line;
with Ada.Strings.Fixed;

with OCI.Thick.Connections;
with OCI.Thick.Statements;
with OCI.Thick.Strings;

with Utils;

procedure RunPLSQL is

   use OCI.Thick;
   use Ada.Command_Line;

   Connect : Connections.Connection;

   Stmt : Statements.Statement;
   
   type String_Access is access all String;
   
   type Variable_List;
   
   type Variable_Access is access all Variable_List;

   type Variable_List is record
      Name  : String_Access;
      Value : String_Access;
      Variable : Strings.Variable;
      Next : Variable_Access;
   end record;
   
   List : Variable_Access;

   Connected : Boolean := False;

   Temp_List : Variable_Access;

   procedure Print_Usage is
   begin
      Ada.Text_IO.Put_Line (
          "usage: runplsql <user/password@tns_name> <plsql_file_name>"
          & " [name=value] ...");
   end Print_Usage;
   
   procedure Split_Name_Value (Data : String;
       Name, Value : out String_Access)
   is
      Del_Idx : Natural := Ada.Strings.Fixed.Index (Data, "=");
   begin
      if Del_Idx = 0 then
         return;
      end if;
      Name := new String'(Data (Data'First .. Del_Idx - 1));
      Value := new String'(Data (Del_Idx + 1 .. Data'Last));
   end Split_Name_Value;

begin
   if Argument_Count < 2 then
      Print_Usage;
      return;
   end if;

   for i in 3 .. Argument_Count loop

      Temp_List := List;
      List := new Variable_List;

      Split_Name_Value (Argument (i),       
         List.Name, List.Value);
      if List.Name = null then
         Print_Usage;
         return;
      end if;

      List.Next  := Temp_List;
      Strings.Set_Value (List.Variable, List.Value.all);

      Ada.Text_IO.Put_Line (List.Name.all & '=' & List.Value.all);

   end loop;

   Connect := Connections.Logon (Argument (1));
   
   Ada.Text_IO.Put_Line ("Connected.");

   Connected := True;

   Stmt := Statements.Prepare (Connect,
      Utils.File_To_String (Argument (2)));

   Ada.Text_IO.Put_Line ("Prepared.");

   Temp_List := List;
   while Temp_List /= null loop
      Strings.Bind (Stmt,
         Temp_List.Variable,
         Temp_List.Name.all);
      Temp_List := Temp_List.Next;
   end loop;

   Ada.Text_IO.Put_Line ("Binded.");
   
   Statements.Execute (Stmt);
   
   Ada.Text_IO.Put_Line ("Executed.");
   
   Connections.Commit (Connect);
   
   Ada.Text_IO.Put_Line ("Commited.");

   Temp_List := List;
   while Temp_List /= null loop
      declare
         Value : String := Strings.Get_Value (Temp_List.Variable);
      begin
         if Value /= Temp_List.Value.all then
            Ada.Text_IO.Put_Line (Temp_List.Name.all & '=' & Temp_List.Value.all);
         end if;
      end;
      Temp_List := Temp_List.Next;
   end loop;
   
   Ada.Text_IO.Put_Line ("Done.");

exception
   when LIB_ERROR =>

      if Connected then
         Connections.Rollback (Connect);
      end if;   

      raise;
end RunPLSQL;